import * as React from 'react';
import * as classnames from 'classnames';
import * as moment from 'moment';
import Tooltip from '@material-ui/core/Tooltip/Tooltip';
import FlightLand from '@material-ui/icons/FlightLand';

import SegmentModel from '../../schemas/Segment';
import { declension, fixImageURL } from '../../utils';
import { i18n } from '../../i18n';
import { BaggageInfo } from './BaggageInfo';

interface Props {
	segment: SegmentModel;
	nemoURL: string;
	renderAdditionalBlock?: () => React.ReactNode;
	isProMode?: boolean;
}

class Segment extends React.Component<Props> {
	renderLogo(): React.ReactNode {
		const segment = this.props.segment;

		return this.props.segment.airline.logoIcon
			? <img className="flight-details-segment-logo__image" title={segment.airline.name} src={fixImageURL(segment.airline.logoIcon, this.props.nemoURL)}/>
			: <div className="flight-details-segment-logo__text">{segment.airline.name}</div>;
	}

	renderStopoverInfo(): string {
		const stopovers = this.props.segment.stopPoints;
		let stopoverInfo = i18n('results-flight-stopoverTitle_one');

		if (stopovers.length) {
			stopovers.forEach((stopover, index) => {
				const delim = index > 0 ? (index === stopovers.length - 1 ? `${i18n('utils-join-and')} ` :  ', ') : '';

				stopoverInfo += `${delim} ${declension(stopover.city ? stopover.city.name : stopover.airport.name)}`;
			});

			return stopoverInfo;
		}

		return null;
	}

	render(): React.ReactNode {
		const segment = this.props.segment;
		const totalFlightTime = segment.flightTime + segment.waitingTime;
		const totalFlightTimeHuman = moment.duration(totalFlightTime, 'seconds').format(`d [${i18n('utils-dates-d')}] h [${i18n('utils-dates-h')}] m [${i18n('utils-dates-m')}]`);
		const hasTransfer = !!segment.prevSegment;
		const waitingTime = moment.duration(segment.prevSegment.waitingTime, 'seconds').format(`d [${i18n('utils-dates-d')}] h [${i18n('utils-dates-h')}] m [${i18n('utils-dates-m')}]`);
		const arrivalAtNextDay = segment.depDate.date() !== segment.arrDate.date();
		const arrivalAtNextDayTitle = arrivalAtNextDay ? <span className="tooltip">{i18n('results-flight-nextDayArrival').replace('[%-date-%]', segment.arrDate.format('DD MMMM'))}</span> : '';

		return <>
			<div className="flight-details-segment">
				{hasTransfer ? (
					<div className="flight-details-segment-transfer">
						<span className="flight-details-segment-transfer-icon"/> {i18n('results-flight-transferInTitle')} {declension(segment.prevSegment.arrAirport.city.name)}, {waitingTime}
					</div>
				) : null}

				<div className="flight-details-segment__wrapper">
					<div className="flight-details-segment__left">
						<div className="flight-details-segment-logo">
							{this.renderLogo()}
						</div>

						<div className="flight-details-segment-stage flight-details-segment-stage_departure">
							<div className="flight-details-segment-stage__time">
								{segment.depDate.format('HH:mm')}
							</div>

							<div className="flight-details-segment-stage__location" title={segment.depAirport.name}>
								{segment.depAirport.IATA}
							</div>
						</div>

						<div className="flight-details-segment-stage-routeInfo">
							<div className="flight-details-segment-stage-routeInfo__arrow"/>
							<span className="flight-details-segment-stage-routeInfo__flightTime">{totalFlightTimeHuman}</span>
						</div>

						<div className="flight-details-segment-stage flight-details-segment-stage_arrival">
							<Tooltip title={arrivalAtNextDayTitle} placement="top">
								<div className={classnames('flight-details-segment-stage__time', { 'flight-details-segment-stage__time_warning': arrivalAtNextDay })}>
									{segment.arrDate.format('HH:mm')}
								</div>
							</Tooltip>

							<div className="flight-details-segment-stage__location" title={segment.arrAirport.name}>
								{segment.arrAirport.IATA}
							</div>
						</div>
					</div>

					<div className="flight-details-segment__middle">
						{segment.stopPoints && segment.stopPoints.length && <div className="flight-summary-stopovers">
							<FlightLand/>

							<div className="flight-summary-stopovers__label">{this.renderStopoverInfo()}</div>
						</div>}

						<div><strong>{segment.airline.IATA}-{segment.flightNumber}</strong>, {segment.aircraft.name}</div>

						{this.props.isProMode && <div>
							{segment.bookingClass.serviceName} ({segment.bookingClass.serviceCode})
						</div>}

						{segment.airline.IATA !== segment.operatingAirline.IATA && <div className="flight-details-segment-operatingAirline">
							{`${i18n('results-flight-operatingCompany')}: ${segment.operatingAirline.name}`}
						</div>}

						<BaggageInfo segment={segment}/>
					</div>

					<div className="flight-details-segment__right">

					</div>
				</div>
			</div>

			{this.props.renderAdditionalBlock ? this.props.renderAdditionalBlock() : null}
		</>;
	}
}

export default Segment;
