import * as React from 'react';
import * as classnames from 'classnames';
import Tooltip from '@material-ui/core/Tooltip/Tooltip';
import StarIcon from '@material-ui/icons/Stars';
import autobind from 'autobind-decorator';

import Flight from './Flight';
import { selectFlight } from '../store/selectedFlights/actions';
import SelectedFlight from '../schemas/SelectedFlight';
import FlightModel from '../models/Flight';
import Money from '../schemas/Money';
import { i18n } from '../i18n';
import { Currency } from '../enums';
import Price from './Price';
import CurrencyRates from '../schemas/CurrencyRates';

interface Props {
	flight: FlightModel;
	replacementFlight: FlightModel;
	selectFlight: typeof selectFlight;
	showPricePrefix: boolean;
	replacement: SelectedFlight;
	totalPrice: Money;
	currentLegId: number;
	nemoURL: string;
	isHidden: boolean;
	isFirstVisible: boolean;
	currency: Currency;
	rates: CurrencyRates;
	isLastLeg?: boolean;
	isProMode: boolean;
}

class ResultsFlight extends React.Component<Props> {
	shouldComponentUpdate(nextProps: Props): boolean {
		return (
			this.props.flight.id !== nextProps.flight.id ||
			this.props.totalPrice !== nextProps.totalPrice ||
			this.props.isHidden !== nextProps.isHidden ||
			this.props.isFirstVisible !== nextProps.isFirstVisible ||
			this.props.replacement !== nextProps.replacement
		);
	}

	@autobind
	onAction(event: React.MouseEvent<HTMLDivElement>): void {
		event.stopPropagation();
		event.preventDefault();

		this.props.selectFlight(this.props.replacement, this.props.currentLegId);
	}

	@autobind
	renderActionBlock(): React.ReactNode {
		const { flight, replacement, totalPrice, currentLegId, showPricePrefix, currency, rates } = this.props;
		const price = replacement ? replacement.price : flight.totalPrice;
		let buttonText;

		if (totalPrice) {
			buttonText = <Price currency={currency} rates={rates} price={totalPrice}/>;
		}
		else {
			buttonText = <span>{i18n('results-flight-buyTitle')}</span>;
		}

		const block = (
			<div className="flight-summary__right">
				<div className="flight-summary-price">
					<div className={classnames('flight-summary-price__amount', { 'flight-summary-price__amount_profitable': price.amount < 0 })}>
						{showPricePrefix && <span className="flight-summary-price__amount-prefix">{i18n('utils-pre-from')}</span>}

						<Price currency={currency} rates={rates} withPlus={currentLegId !== 0} price={price}/>
					</div>

					{price.amount < 0 && (
						<div className="flight-summary-price-profitMark">
							<div className="flight-summary-price-profitMark__icon">
								<StarIcon/>
							</div>

							<span className="flight-summary-price-profitMark__text">{i18n('results-flight-profitable-title')}</span>
						</div>
					)}

					{currentLegId === 0 && (
						<div className="flight-summary-price__route">
							{i18n('results-flight-wholeFlightTitle')}
						</div>
					)}
				</div>

				<div className="flight-summary__action">
					<div className="flight-summary-buy" onClick={this.onAction}>{buttonText}</div>
					{this.props.isProMode && <div className="flight-summary-service">{flight.service}</div>}
				</div>
			</div>
		);

		if (price.amount < 0) {
			const title = (
				<span className="tooltip">
					{i18n('results-flight-profitable-tooltip_1')}<br/>
					{i18n('results-flight-profitable-tooltip_2')}
				</span>
			);

			return <Tooltip placement="top" title={title}>{block}</Tooltip>;
		}
		else {
			return block;
		}
	}

	render(): React.ReactNode {
		const className = classnames('flight', {
			flight_hidden: this.props.isHidden,
			flight_visible: !this.props.isHidden,
			flight_firstVisible: this.props.isFirstVisible
		});

		return (
			<Flight
				{...this.props}
				className={className}
				showFilters={true}
				showFareRules={true}
				renderActionBlock={this.renderActionBlock}
				replacedFlightInfo={this.props.replacement.isRT ? this.props.replacement.flightInfo : null}
				totalPrice={this.props.totalPrice}
			/>
		);
	}
}

export default ResultsFlight;
