import { createSelector } from 'reselect';
import Flight from '../models/Flight';
import { getListOfSelectedCodes, ListOfSelectedCodes } from './filters/selectors';
import {
	getAllFlights,
	getFlightsForCurrentLeg,
	getMinPricesByLegs,
	getMinTotalPossiblePricesByLegs
} from './flights/selectors';
import * as Sorting from './sorting/selectors';
import { sortingFunctionsMap } from './sorting/selectors';
import { FlightsRTState } from './flightsRT/reducers';
import Money from '../schemas/Money';
import { getCurrentLegId, isFirstLeg, isLastLeg } from './currentLeg/selectors';
import * as FareFamilies from './fareFamilies/selectors';
import {
	getAirlinesIATA,
	getSelectedFlights,
	getSelectedFlightsIds,
	getTotalPriceOfSelectedFlights,
	isSelectionComplete
} from './selectedFlights/selectors';
import { getFlightsRT } from './flightsRT/selectors';
import {
	calculatePricingInfo,
	MAX_VISIBLE_FLIGHTS,
	MIN_TRANSFER_TIME_IN_HOURS_BETWEEN_FLIGHTS,
	UID_LEG_GLUE
} from '../utils';
import { Currency, Route } from '../enums';
import { AdvancedFlightInfo, FlightsReplacement } from '../schemas/SelectedFlight';
import { SelectedFlightsState } from './selectedFlights/reducers';
import { FlightsState } from './flights/reducers';
import { SortingState } from './sorting/reducers';
import { FareFamiliesCombinationsState } from './fareFamilies/fareFamiliesCombinations/reducers';
import { getShowAllFlights } from './showAllFlights/selectors';
import {
	getFilteredArrivalTimeIntervals,
	getFilteredDepartureTimeIntervals,
	getTimeIntervalForDate
} from './filters/time/selectors';
import { RootState } from './reducers';
import { getFilteredArrivalAirports, getFilteredDepartureAirports } from './filters/airports/selectors';
import { getFilteredAirlines } from './filters/airlines/selectors';
import { getIsRTMode } from './fareFamilies/isRTMode/selectors';
import { CombinationsPrices } from '../schemas/FareFamiliesCombinations';
import { getCurrency } from './currency/selectors';
import { PricesByLegs } from '../schemas/PricesByLegs';
import { AdvancedFlightInfoByFlights } from '../schemas/PricingInfo';
import { PassengerType, SearchInfo, SearchInfoPassenger } from '@nemo.travel/search-widget';
import * as moment from 'moment';

export interface PricesByFlights {
	[flightId: string]: Money;
}

export interface FlightsVisibility {
	[flightId: string]: boolean;
}

export interface FilterSelectors {
	selectedAirlines: ListOfSelectedCodes;
	selectedDepartureAirports: ListOfSelectedCodes;
	selectedArrivalAirports: ListOfSelectedCodes;
	selectedDepartureTimeIntervals: ListOfSelectedCodes;
	selectedArrivalTimeIntervals: ListOfSelectedCodes;
	directOnly: boolean;
	flightSearch: string;
	comfortable: boolean
}

/**
 * Calculating total price.
 */
export const getTotalPrice = createSelector(
	[
		getAllFlights,
		getSelectedFlightsIds,
		isSelectionComplete,
		FareFamilies.getSelectedCombinations,
		FareFamilies.getFareFamiliesCombinations,
		getMinPricesByLegs,
		getMinTotalPossiblePricesByLegs,
		getIsRTMode,
		getCurrency
	],
	(
		allFlights: FlightsState,
		selectedFlightsIds: SelectedFlightsState,
		selectionComplete: boolean,
		selectedCombinations: FareFamilies.SelectedCombinations,
		combinations: FareFamiliesCombinationsState,
		minPricesByLegs: PricesByLegs,
		minTotalPossiblePricesByLegs: PricesByLegs,
		isRTMode: boolean,
		currency: Currency
	): Money => {
		let RTFound = false;
		const totalPrice: Money = {
			amount: 0,
			currency: currency
		};

		if (selectionComplete && isRTMode) {
			let prices: CombinationsPrices;
			const combinationsParts: string[] = [];

			for (const legId in selectedFlightsIds) {
				if (selectedFlightsIds.hasOwnProperty(legId)) {
					if (selectedCombinations[legId] && combinations[legId]) {
						if (!prices) {
							prices = combinations[legId].combinationsPrices;
						}

						combinationsParts.push(selectedCombinations[legId]);
					}
				}
			}

			const resultingCombination = combinationsParts.join('_');

			if (prices && prices.hasOwnProperty(resultingCombination)) {
				totalPrice.amount = prices[resultingCombination].amount;
			}
		}
		else {
			for (const legId in selectedFlightsIds) {
				if (selectedFlightsIds.hasOwnProperty(legId)) {
					const intLegId = parseInt(legId);
					const flightId = selectedFlightsIds[legId].newFlightId;

					// If main flights have been successfully selected,
					// then it's time to choose alternative flights (fare families).
					if (selectionComplete && selectedCombinations[legId] && combinations[legId]) {
						const combination = selectedCombinations[legId];

						// Check if selected fare families combination is valid and has its own price.
						if (combinations[legId].combinationsPrices.hasOwnProperty(combination)) {
							totalPrice.amount += combinations[legId].combinationsPrices[combination].amount;
						}
					}
					else {
						// Get flight and add its price to the total sum.
						if (allFlights.hasOwnProperty(flightId)) {
							const flight = allFlights[flightId];

							if (!RTFound) {
								totalPrice.amount += flight.totalPrice.amount;
							}

							if (flight.isRT || flight.id.indexOf('/') !== -1) {
								RTFound = true;
							}
						}
					}

					if (!RTFound && !selectionComplete && minTotalPossiblePricesByLegs[intLegId]) {
						totalPrice.amount += minTotalPossiblePricesByLegs[intLegId].amount;
					}
				}
			}
		}

		return totalPrice;
	}
);

export const getPricesForCurrentLeg = createSelector(
	[
		getFlightsForCurrentLeg,
		getFlightsRT,
		isLastLeg,
		getSelectedFlights,
		getTotalPriceOfSelectedFlights
	],
	(
		flights: Flight[],
		flightsRT: FlightsRTState,
		isLastLeg: boolean,
		selectedFlights: Flight[],
		totalPriceOfSelectedFlights: number
	): FlightsReplacement => {
		const result: FlightsReplacement = {};
		const selectedUID = selectedFlights.map(flight => flight.uid).join(UID_LEG_GLUE);

		flights.forEach(flight => {
			let newFlightId = flight.id;
			let price = flight.totalPrice;
			let isRT = false;
			let flightInfo: AdvancedFlightInfo = null;
			let pricingInfo = flight.pricingInfo;
			const candidateUID = selectedUID ? `${selectedUID}|${flight.uid}` : flight.uid;
			const possibleTotalPrice = totalPriceOfSelectedFlights + price.amount;
			const originalFlightId = flight.id;

			if (isLastLeg) {
				// Sometimes RT flight could be cheaper than OW+OW flight.
				// Loop through all RT flights and try to find one that is cheaper than currently selected combination.
				for (const uid in flightsRT) {
					if (flightsRT.hasOwnProperty(uid)) {
						const RTFlight: Flight = flightsRT[uid];

						// RT UID example: 'AY-720_1pc_AY-1071_1pc|AY-1076_1pc_AY-719_1pc'
						if (uid.startsWith(candidateUID) && RTFlight.totalPrice.amount < possibleTotalPrice) {
							price = RTFlight.totalPrice;
							newFlightId = RTFlight.id;
							isRT = true;
							pricingInfo = RTFlight.pricingInfo;
							flightInfo = {
								pricingInfo: RTFlight.pricingInfo,
								expectedNumberOfTickets: RTFlight.expectedNumberOfTickets,
								pricingDebugLinks: RTFlight.pricingDebugLink ? [RTFlight.pricingDebugLink] : []
							};

							break;
						}
					}
				}
			}

			result[flight.id] = {
				price,
				isRT,
				originalFlightId,
				newFlightId,
				flightInfo
			};
		});

		return result;
	}
);

export const getRelativePrices = createSelector(
	[
		getAllFlights,
		getPricesForCurrentLeg,
		getMinPricesByLegs,
		getTotalPrice,
		getMinTotalPossiblePricesByLegs,
		getCurrentLegId
	],
	(
		allFlights: FlightsState,
		prices: FlightsReplacement,
		minPrices: PricesByLegs,
		totalPrice: Money,
		minTotalPossiblePricesByLegs: PricesByLegs,
		legId: number
	): FlightsReplacement => {
		const minPriceOnCurrentLeg = minPrices[legId] ? minPrices[legId].amount : 0;
		const minTotalPriceForNextLegs = minTotalPossiblePricesByLegs[legId] ? minTotalPossiblePricesByLegs[legId].amount : 0;
		const result: FlightsReplacement = {};

		for (const flightId in prices) {
			if (prices.hasOwnProperty(flightId) && allFlights.hasOwnProperty(flightId)) {
				const priceCandidate = prices[flightId];
				let amount: number;

				if (priceCandidate.isRT) {
					// Hurray! We have found cheap RT flight.
					amount = priceCandidate.price.amount - totalPrice.amount;
				}
				else {
					// We haven't found any cheap RT flights :(
					amount = priceCandidate.price.amount + (legId === 0 ? minTotalPriceForNextLegs : -(minPriceOnCurrentLeg));
				}

				result[flightId] = {
					...priceCandidate,
					price: {
						amount,
						currency: priceCandidate.price.currency
					}
				};
			}
		}

		return result;
	}
);

/**
 * List of total prices for current leg.
 */
export const getTotalPrices = createSelector(
	[
		getCurrentLegId,
		isFirstLeg,
		getSelectedFlights,
		getAllFlights,
		getPricesForCurrentLeg,
		getMinTotalPossiblePricesByLegs,
		getTotalPriceOfSelectedFlights
	],
	(
		legId: number,
		isFirstLeg: boolean,
		selectedFlights: Flight[],
		allFlights: FlightsState,
		replacements: FlightsReplacement,
		minTotalPossiblePricesByLegs: PricesByLegs,
		previousAddition: number
	): PricesByFlights => {
		const result: PricesByFlights = {};

		if (!isFirstLeg) {
			const nextAddition = minTotalPossiblePricesByLegs[legId].amount;

			for (const flightId in replacements) {
				if (replacements.hasOwnProperty(flightId) && allFlights.hasOwnProperty(flightId)) {
					if (replacements[flightId].isRT) {
						result[flightId] = replacements[flightId].price;
					}
					else {
						const flight = allFlights[flightId];

						result[flight.id] = {
							amount: previousAddition + flight.totalPrice.amount + nextAddition,
							currency: flight.totalPrice.currency
						};
					}
				}
			}
		}

		return result;
	}
);

export const getPricesForSelectedFlights = createSelector(
	[getSelectedFlights, getMinPricesByLegs, getMinTotalPossiblePricesByLegs],
	(flights: Flight[], minPrices: PricesByLegs, minTotalPossiblePricesByLegs: PricesByLegs): FlightsReplacement => {
		const result: FlightsReplacement = {};

		flights.forEach(flight => {
			const legId = flight.legId;
			const minPriceOnCurrentLeg = minPrices[legId] ? minPrices[legId].amount : 0;
			const minTotalPriceForNextLegs = minTotalPossiblePricesByLegs[legId] ? minTotalPossiblePricesByLegs[legId].amount : 0;

			result[flight.id] = {
				newFlightId: flight.id,
				originalFlightId: flight.id,
				isRT: flight.isRT,
				price: {
					amount: flight.totalPrice.amount + (legId === 0 ? minTotalPriceForNextLegs : -(minPriceOnCurrentLeg)),
					currency: flight.totalPrice.currency
				}
			};
		});

		return result;
	}
);

export const getSelectedAirlinesList = createSelector([getFilteredAirlines], getListOfSelectedCodes);

export const getSelectedDepartureAirportsList = createSelector([getFilteredDepartureAirports], getListOfSelectedCodes);
export const getSelectedArrivalAirportsList = createSelector([getFilteredArrivalAirports], getListOfSelectedCodes);

export const getSelectedDepartureTimeIntervals = createSelector([getFilteredDepartureTimeIntervals], getListOfSelectedCodes);
export const getSelectedArrivalTimeIntervals = createSelector([getFilteredArrivalTimeIntervals], getListOfSelectedCodes);

export const getIsDirectOnly = (state: RootState): boolean => state.filters.directOnly;
export const getFlightSearch = (state: RootState): string => state.filters.flightSearch.search;
export const getIsComfortable = (state: RootState): boolean => state.filters.comfortable;

export const filtersConfig = createSelector(
	[
		getSelectedAirlinesList,
		getSelectedDepartureAirportsList,
		getSelectedArrivalAirportsList,
		getSelectedDepartureTimeIntervals,
		getSelectedArrivalTimeIntervals,
		getIsDirectOnly,
		getFlightSearch,
		getIsComfortable
	],
	(
		selectedAirlines: ListOfSelectedCodes,
		selectedDepartureAirports: ListOfSelectedCodes,
		selectedArrivalAirports: ListOfSelectedCodes,
		selectedDepartureTimeIntervals: ListOfSelectedCodes,
		selectedArrivalTimeIntervals: ListOfSelectedCodes,
		directOnly: boolean,
		flightSearch: string,
		comfortable: boolean
	) => {
		return {
			selectedAirlines,
			selectedDepartureAirports,
			selectedArrivalAirports,
			selectedDepartureTimeIntervals,
			selectedArrivalTimeIntervals,
			directOnly,
			flightSearch,
			comfortable
		};
	}
);

export const isAnyFilterApplied = createSelector(
	[filtersConfig],
	({
		 selectedAirlines,
		 selectedDepartureAirports,
		 selectedArrivalAirports,
		 selectedDepartureTimeIntervals,
		 selectedArrivalTimeIntervals,
		 directOnly,
		 flightSearch,
		 comfortable
	}: FilterSelectors): boolean => {
		return directOnly || comfortable || !!flightSearch ||
			!!Object.keys(selectedAirlines).length ||
			!!Object.keys(selectedArrivalAirports).length ||
			!!Object.keys(selectedDepartureAirports).length ||
			!!Object.keys(selectedArrivalTimeIntervals).length ||
			!!Object.keys(selectedDepartureTimeIntervals).length;
	}
);

/**
 * Get sorted list of flights on the current leg.
 */
export const getSortedFlights = createSelector(
	[getFlightsForCurrentLeg, Sorting.getCurrentSorting, getRelativePrices],
	(flights: Flight[], sorting: SortingState, prices: FlightsReplacement): Flight[] => {
		return flights.slice().sort((a, b) => sortingFunctionsMap[sorting.type](a, b, sorting.direction, prices));
	}
);

/**
 * Get sorted and filtered lsit of flights on the current leg.
 */
export const getFilteredFlights = createSelector(
	[
		getSortedFlights,
		getSelectedFlights,
		isAnyFilterApplied,
		filtersConfig
	],
	(
		flights: Flight[],
		selectedFlights: Flight[],
		filtersAreActive: boolean,
		{
			selectedAirlines,
			selectedDepartureAirports,
			selectedArrivalAirports,
			selectedDepartureTimeIntervals,
			selectedArrivalTimeIntervals,
			directOnly,
			flightSearch,
			comfortable
		}: FilterSelectors
	): Flight[] => {
		// Skip filtering if no filters applied.
		if (!filtersAreActive && !selectedFlights.length) {
			return flights;
		}

		const
			selectedAirlineCodes = comfortable ? getAirlinesIATA(selectedFlights) : {},
			lastSegmentArrAirportIATA = comfortable ? selectedFlights[selectedFlights.length - 1].lastSegment.arrAirport.IATA : null,
			lastSegmentArrivalDateTime = selectedFlights.length ? selectedFlights[selectedFlights.length - 1].lastSegment.arrDate : null;

		return flights.filter(flight => {
			const firstSegment = flight.segments[0];
			const lastSegment = flight.segments[flight.segments.length - 1];

			// Filter direct flights.
			if (directOnly && flight.segments.length !== 1) {
				return false;
			}

			// Filter by searchIndex.
			if (flightSearch && flight.searchIndex.indexOf(flightSearch.toLowerCase()) === -1) {
				return false;
			}

			// Filter by departure airport.
			if (Object.keys(selectedDepartureAirports).length && !(firstSegment.depAirport.IATA in selectedDepartureAirports)) {
				return false;
			}

			// Filter by arrival airport.
			if (Object.keys(selectedArrivalAirports).length && !(lastSegment.arrAirport.IATA in selectedArrivalAirports)) {
				return false;
			}

			// Filter by airline.
			if (Object.keys(selectedAirlines).length && !flight.segments.find(segment => segment.airline.IATA in selectedAirlines)) {
				return false;
			}

			// Filter by departure time.
			if (Object.keys(selectedDepartureTimeIntervals).length && !(getTimeIntervalForDate(firstSegment.depDate) in selectedDepartureTimeIntervals)) {
				return false;
			}

			// Filter by arrival time.
			if (Object.keys(selectedArrivalTimeIntervals).length && !(getTimeIntervalForDate(lastSegment.arrDate) in selectedArrivalTimeIntervals)) {
				return false;
			}

			// Show only `comfortable` flight if checked.
			if (comfortable) {
				if (lastSegmentArrAirportIATA !== flight.firstSegment.depAirport.IATA) {
					return false;
				}

				return flight.segments.find(segment => {
					if (selectedAirlineCodes.hasOwnProperty(segment.airline.IATA)) {
						return true;
					}
				});
			}

			// Next flight must be later by 2 hours than previous flight
			if (
				lastSegmentArrivalDateTime &&
				flight.firstSegment.depDate.diff(lastSegmentArrivalDateTime, 'hour') < MIN_TRANSFER_TIME_IN_HOURS_BETWEEN_FLIGHTS
			) {
				return false;
			}

			return true;
		});
	}
);

export const getVisibleFlights = createSelector(
	[getFilteredFlights, getShowAllFlights],
	(flights: Flight[], showAllFlights: boolean): Flight[] => {
		let results = flights;

		if (!showAllFlights) {
			results = flights.slice(0, MAX_VISIBLE_FLIGHTS);
		}

		return results;
	}
);

export const getVisibleFlightsMap = createSelector(
	[getVisibleFlights],
	(flights: Flight[]) => {
		const result: FlightsVisibility = {};

		flights.forEach(flight => {
			result[flight.id] = true;
		});

		return result;
	}
);

/**
 * Check if there are any visible flights for current leg.
 */
export const hasAnyVisibleFlights = createSelector(
	[getVisibleFlights],
	(flights: Flight[]): boolean => !!flights.length
);

export const hasHiddenFlights = createSelector(
	[getShowAllFlights, getFlightsForCurrentLeg, getVisibleFlights],
	(showAllFlights: boolean, allFlights: Flight[], visibleFlights: Flight[]): boolean => {
		return !showAllFlights && allFlights.length > MAX_VISIBLE_FLIGHTS && !(visibleFlights.length < MAX_VISIBLE_FLIGHTS);
	}
);

export const getRouterLocation = (state: RootState) => state.router.location.pathname;

export const getRoute = createSelector(
	[getRouterLocation],
	(location: string): Route => {
		if (/\/results\/(\d+\/?)+/.test(location)) {
			return Route.ResultsWithIds;
		}
		else if (/\/results\/((?:[A-ZА-Я]{3}[A-ZА-Я]{3}\d{8,16})+)((?:[A-Z]{3}[1-9])+)?((?:-[a-zA-Z=\d\+]+)+)?(?:\/?\?.*)?$/.test(location)) {
			return Route.FastSearch;
		}
		else if (location === '/results') {
			return Route.Results;
		}

		return Route.Initial;
	}
);

const getAdvancedInfoForSelectedFlights = createSelector(
	[getSelectedFlights],
	(selectedFlights: Flight[]): AdvancedFlightInfo => {
		if (selectedFlights.length) {

			const joinedSelectedFlightsAdvancedInfo: AdvancedFlightInfo = {
				pricingInfo: selectedFlights[0].pricingInfo,
				expectedNumberOfTickets: selectedFlights[0].expectedNumberOfTickets,
				pricingDebugLinks: []
			};

			if (selectedFlights[0].pricingDebugLink) {
				joinedSelectedFlightsAdvancedInfo.pricingDebugLinks.push(selectedFlights[0].pricingDebugLink);
			}

			selectedFlights.forEach((flight, index) => {
				if (index > 0) {
					if (flight.pricingInfo.agencyCharge) {
						joinedSelectedFlightsAdvancedInfo.pricingInfo.agencyCharge.amount += flight.pricingInfo.agencyCharge.amount;
					}

					if (joinedSelectedFlightsAdvancedInfo.pricingInfo.subagentProfit) {
						joinedSelectedFlightsAdvancedInfo.pricingInfo.subagentProfit.amount += flight.pricingInfo.subagentProfit.amount;
					}

					if (flight.pricingInfo.agentProfit) {
						joinedSelectedFlightsAdvancedInfo.pricingInfo.agentProfit.amount += flight.pricingInfo.agentProfit.amount;
					}

					joinedSelectedFlightsAdvancedInfo.pricingInfo.fares = joinedSelectedFlightsAdvancedInfo.pricingInfo.fares.concat(flight.pricingInfo.fares);

					if (flight.expectedNumberOfTickets) {
						joinedSelectedFlightsAdvancedInfo.expectedNumberOfTickets += flight.expectedNumberOfTickets;
					}

					if (flight.pricingDebugLink) {
						joinedSelectedFlightsAdvancedInfo.pricingDebugLinks.push(flight.pricingDebugLink);
					}
				}
			});

			joinedSelectedFlightsAdvancedInfo.pricingInfo.fares = calculatePricingInfo(joinedSelectedFlightsAdvancedInfo.pricingInfo.fares);

			return joinedSelectedFlightsAdvancedInfo;
		}

		return null;
	}
);

export const getAdvancedInfoByFlights = createSelector(
	[getAdvancedInfoForSelectedFlights, getSelectedFlights, getFlightsForCurrentLeg, getRelativePrices],
	(advancedInfoForSelectedFlights: AdvancedFlightInfo, selectedFlights: Flight[], currentLegFlights: Flight[], relativePrices: FlightsReplacement): AdvancedFlightInfoByFlights => {
		const pricingInfoByFlights: AdvancedFlightInfoByFlights = {};

		currentLegFlights.forEach(flight => {
			let pricingInfo: AdvancedFlightInfo;

			if (relativePrices[flight.id].isRT) {
				 pricingInfo = relativePrices[flight.id].flightInfo;
			}
			else {
				pricingInfo = {
					pricingInfo: flight.pricingInfo,
					expectedNumberOfTickets: flight.expectedNumberOfTickets,
					pricingDebugLinks: []
				};

				if (advancedInfoForSelectedFlights) {
					// pushing array without references
					pricingInfo.pricingDebugLinks = pricingInfo.pricingDebugLinks.concat(advancedInfoForSelectedFlights.pricingDebugLinks);

					if (advancedInfoForSelectedFlights.pricingInfo.agentProfit) {
						pricingInfo.pricingInfo.agentProfit.amount += advancedInfoForSelectedFlights.pricingInfo.agentProfit.amount;
					}

					if (advancedInfoForSelectedFlights.pricingInfo.subagentProfit) {
						pricingInfo.pricingInfo.subagentProfit.amount += advancedInfoForSelectedFlights.pricingInfo.subagentProfit.amount;
					}

					if (advancedInfoForSelectedFlights.pricingInfo.agencyCharge) {
						pricingInfo.pricingInfo.agencyCharge.amount += advancedInfoForSelectedFlights.pricingInfo.agencyCharge.amount;
					}

					pricingInfo.pricingInfo.fares = pricingInfo.pricingInfo.fares.concat(advancedInfoForSelectedFlights.pricingInfo.fares);

					if (advancedInfoForSelectedFlights.expectedNumberOfTickets) {
						pricingInfo.expectedNumberOfTickets += advancedInfoForSelectedFlights.expectedNumberOfTickets;
					}
				}

				if (flight.pricingDebugLink) {
					pricingInfo.pricingDebugLinks.push(flight.pricingDebugLink);
				}
			}

			pricingInfo.pricingInfo.fares = calculatePricingInfo(pricingInfo.pricingInfo.fares);

			pricingInfoByFlights[flight.id] = pricingInfo;
		});

		return pricingInfoByFlights;
	}
);
